﻿using ProjectAirFighter.DrawningObjects;
using ProjectAirFighter.Generics;
using ProjectAirFighter.MovementStrategy;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ProjectAirFighter
{
    public partial class FormAirplaneCollection : Form
    {
        private readonly AirplanesGenericStorage _storage;

        public FormAirplaneCollection()
        {
            InitializeComponent();
            _storage = new AirplanesGenericStorage(pictureBoxCollection.Width,
                 pictureBoxCollection.Height);
        }
        private void ReloadObjects()
        {
            int index = listBoxStorages.SelectedIndex;

            listBoxStorages.Items.Clear();
            for (int i = 0; i < _storage.Keys.Count; i++)
            {
                listBoxStorages.Items.Add(_storage.Keys[i]);
            }
            if (listBoxStorages.Items.Count > 0 && (index == -1 || index
>= listBoxStorages.Items.Count))
            {
                listBoxStorages.SelectedIndex = 0;
            }
            else if (listBoxStorages.Items.Count > 0 && index > -1 &&
            index < listBoxStorages.Items.Count)
            {
                listBoxStorages.SelectedIndex = index;
            }
            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }
            var obj = _storage[listBoxStorages.SelectedItem.ToString() ??
            string.Empty];
            if (obj == null)
            {
                return;
            }
            pictureBoxCollection.Image = obj.ShowAirplanes();

        }
        private void ButtonAddObject_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBoxStorageName.Text))
            {
                MessageBox.Show("Не все данные заполнены", "Ошибка",
                MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            _storage.AddSet(textBoxStorageName.Text);
            ReloadObjects();

        }
        private void deleteAirplaneButton_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }
            var obj = _storage[listBoxStorages.SelectedItem.ToString() ??
            string.Empty];
            if (obj == null)
            {
                return;
            }
            if (MessageBox.Show("Удалить объект?", "Удаление",
            MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }
            int pos = Convert.ToInt32(maskedTextBoxNumber.Text);
            if (obj - pos != null)
            {
                MessageBox.Show("Объект удален");
                pictureBoxCollection.Image = obj.ShowAirplanes();
            }
            else
            {
                MessageBox.Show("Не удалось удалить объект");
            }
        }

        private void updateCollectionButton_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }
            var obj = _storage[listBoxStorages.SelectedItem.ToString() ??
            string.Empty];
            if (obj == null)
            {
                return;
            }
            pictureBoxCollection.Image = obj.ShowAirplanes();

        }

        private void addAirplaneButton_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }
            var obj = _storage[listBoxStorages.SelectedItem.ToString() ??
            string.Empty];
            if (obj == null)
            {
                return;
            }
            FormAirplaneConfig form = new();
            form.Show();
            Action<DrawningAirplane>? airplaneDelegate = new((m) =>
            {
                bool q = (obj + m);
                if (q)
                {
                    MessageBox.Show("Объект добавлен");
                    pictureBoxCollection.Image = obj.ShowAirplanes();
                }
                else
                {
                    MessageBox.Show("Не удалось добавить объект");
                }
            });
            form.AddEvent(airplaneDelegate);
        }
        private void listBoxStorages_SelectedIndexChanged(object sender, EventArgs e)
        {
            pictureBoxCollection.Image =
_storage[listBoxStorages.SelectedItem?.ToString() ?? string.Empty]?.ShowAirplanes();
        }

        private void ButtonDelObject_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }
            if (MessageBox.Show($"Удалить объект{listBoxStorages.SelectedItem}?", "Удаление", MessageBoxButtons.YesNo,
MessageBoxIcon.Question) == DialogResult.Yes)
            {
                _storage.DelSet(listBoxStorages.SelectedItem.ToString()
                ?? string.Empty);
                ReloadObjects();
            }

        }

    }
}
