﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectAirFighter.Generics
{
    internal class SetGeneric<T>
        where T : class
    {
        private readonly List<T?> _places;

        public int Count => _places.Count;

        private readonly int _maxCount;
        public SetGeneric(int count)
        {
            _maxCount = count;
            _places = new List<T?>(count);
        }

        public bool Insert(T airplane)
        {
            if (_places.Count == _maxCount)
                return false;
            Insert(airplane, 0);
            return true;

        }

        public bool Insert(T airplane, int position)
        {
            if (!(position >= 0 && position <= Count && _places.Count < _maxCount))
                return false;
            _places.Insert(position, airplane);
            return true;
        }
        public bool Remove(int position)
        {
            if (!(position >= 0 && position < Count))
                return false;
            _places.RemoveAt(position);
            return true;
        }
        public T? this[int position]
        {
            get
            {
                if (!(position >= 0 && position < Count))
                    return null;
                return _places[position];
            }
            set
            {
                if (!(position >= 0 && position < Count && _places.Count < _maxCount))
                    return;
                _places.Insert(position, value);
                return;
            }
        }

        public IEnumerable<T?> GetAirplanes(int? maxAirplanes = null)
        {
            for (int i = 0; i < _places.Count; ++i)
            {
                yield return _places[i];
                if (maxAirplanes.HasValue && i == maxAirplanes.Value)
                {
                    yield break;
                }
            }
        }

    }
}
